#include <Arduino.h>
#include <midea_ir.h>
#include <WiFi.h>
#include <DFRobot_DS1307.h>
#include <MPython_NtpTime.h>
#include <DFRobot_DF1101S.h>

DFRobot_DF1101S df1101s;
MideaIR ir;
MPython_NtpTime ntptime;
DFRobot_DS1307 RTC;

const uint8_t MIDEA_RMT_CHANNEL = 0;//红外遥控编码
const uint8_t MIDEA_TX_PIN = D3;   //红外遥控引脚

const uint8_t LIGHT_PIN = A0;//光照
const uint8_t SOUND_PIN = A1;//声音

const uint8_t RX_PIN = D7;//语音模块引脚
const uint8_t TX_PIN = D8;//

const char* ssid     = "dfrobotGuest";
const char* password = "dfrobot2020";

bool wifi_state = false;
// 校正时间
void correct_time () {
  if (WiFi.status() != WL_CONNECTED) init_my_wifi();
  if (WiFi.status() != WL_CONNECTED) {
     Serial.println("correct_time error : wifi");
   return; 
  }
  // 获取ntp时间
  ntptime.setNtpTime(ntptime.UTCEast8_t, "ntp.ntsc.ac.cn");
  char str[100] = {};
  //Serial.println(sprintf(str, "time: %d-%d-%d %d:%d:%d", constrain(ntptime.localTime(ntptime.Year), 2000, 2099), constrain(ntptime.localTime(ntptime.Month), 1, 12), constrain(ntptime.localTime(ntptime.Date), 1, 31), constrain(ntptime.localTime(ntptime.Hour), 0, 23), constrain(ntptime.localTime(ntptime.Minute), 0, 59), constrain(ntptime.localTime(ntptime.Second), 0, 59)));
  Serial.println(ntptime.localTime(ntptime.Year));
  Serial.println(ntptime.localTime(ntptime.Month));
  Serial.println(ntptime.localTime(ntptime.Date));
  Serial.println(ntptime.localTime(ntptime.Hour));
  Serial.println(ntptime.localTime(ntptime.Minute));
  Serial.println(ntptime.localTime(ntptime.Second));
   Serial.println("-------------------");
  // 写入rtc模块
  RTC.setTime(constrain(ntptime.localTime(ntptime.Year), 2000, 2099), constrain(ntptime.localTime(ntptime.Month), 1, 12), constrain(ntptime.localTime(ntptime.Date), 1, 31), constrain(ntptime.localTime(ntptime.Hour), 0, 23), constrain(ntptime.localTime(ntptime.Minute), 0, 59), constrain(ntptime.localTime(ntptime.Second), 0, 59));
  }

void ir_send_close () {//遥控空调
  for (int i = 0; i < 20; i++){
    digitalWrite(D4, i%2==0?HIGH:LOW);
     ir.enabled = false;
      midea_ir_send(&ir);
      delay(100);
  }
  digitalWrite(D4, wifi_state?HIGH:LOW);
  Serial.println("send!!!");
}
void init_my_wifi () {
   Serial.println();
    Serial.println();
    Serial.print("Connecting to ");
    Serial.println(ssid);
    // wifi初始化
    WiFi.begin(ssid, password);
    int n = 0;
    while (WiFi.status() != WL_CONNECTED) {
        delay(500);
        Serial.print(".");
        digitalWrite(D4, n%2==0?HIGH:LOW);
        if (n++ > 100) {
          wifi_state = false;
          digitalWrite(D4, LOW);
          return;  
        }
    }
    wifi_state = true;
    digitalWrite(D4, HIGH);
    Serial.println("");
    Serial.println("WiFi connected");
    Serial.println("IP address: ");
    Serial.println(WiFi.localIP());
    Serial.println(millis());
}

void showtime()//时间
{
    Serial.print( RTC.getTime(RTC.DS1307_YR) );
    Serial.print("-");    Serial.print( RTC.getTime(RTC.DS1307_MTH));
    Serial.print("-");    Serial.println( RTC.getTime(RTC.DS1307_DATE));
    /*HOUR  MIN SEC */
    Serial.print(  RTC.getTime(RTC.DS1307_HR) );
    Serial.print(":");    Serial.print( RTC.getTime(RTC.DS1307_MIN));
    Serial.print(":");    Serial.println( RTC.getTime(RTC.DS1307_SEC));
    /********WEEK*****/
    Serial.println("**********");
}

void init_player () {     //播放语音
  Serial1.begin(115200,SERIAL_8N1,RX_PIN,TX_PIN);
  while(!df1101s.begin(Serial1)){
    Serial.println("Init failed, please check the wire connection!");//初始化失败，检查连线
    delay(1000);
  }
  df1101s.setVol(5);//设置音量
  df1101s.switchFunction(df1101s.MUSIC);//工作模式（音乐）
  df1101s.setPlayMode(df1101s.SINGLE);//播放模式（一个音频）
}
void setup()
{
  Serial.begin(115200);
  pinMode(D2, INPUT);//按键
  pinMode(D4, OUTPUT);//灯
  RTC.begin(); // 实时时钟模块初始化
     midea_ir_init(&ir, MIDEA_RMT_CHANNEL, MIDEA_TX_PIN);// 红外初始化
    // set state to disabled
    
    delay(500);
    correct_time();//校正时间
    init_player();//播放语音
}

void my_log(){
    showtime();//时间
}

unsigned long last_time = 0;  //上次
unsigned long last_button_time = 0;//按键1s 测试
bool button_press = false;   //按键
int state = -1;   //
int light_count = 0;  //光
int sound_count = 0;  //声音
int sound_count_nobody = 10*60*1000/5;
int light_count_nobody = 10*60*1000/5;
bool ask_flag = false; //询问判断
bool closed = false;  //光判断
unsigned long ask_time = 0;
unsigned long ask_time_1 = 0;
void loop()
{
  unsigned long time = millis();//读取系统运行时间
  // 按键
  if (digitalRead(D2)) {
    if (!button_press){
      button_press = true;
      last_button_time = time;
    }
  } else if (button_press) {
    button_press = false;
    // 按1秒以内 手动设定RTC模块时间为23:59:30, 等待测试
    if ((time - last_button_time) >= 20 && (time - last_button_time) < 2000){
      //ir_send_close();
      state = -1;
      closed = false;
      ask_flag = false;
      light_count = 0;
      sound_count = 0;
      light_count_nobody = 10*1000/5;// 十秒
      sound_count_nobody = 10*1000/5;// 十秒
      RTC.setTime(2022, 3, 11, 23, 59, 50);
    }
    // 按3-5秒 手动设定RTC模块时间为19:30, 检测环境光
    if ((time - last_button_time) >= 2000 && (time - last_button_time) < 5000){
      state = -1;
      closed = false;
      ask_flag = false;
      light_count = 0;
      sound_count = 0;
      light_count_nobody = 10*1000/5;// 十秒
      sound_count_nobody = 10*1000/5;// 十秒
      RTC.setTime(2022, 3, 11, 19, 30, 0);
    }
    // 按3秒 询问结束
    if ((time - last_button_time) >= 5000 && (time - last_button_time) < 3500){
      state = -1;
      closed = false;
      ask_flag = false;
      light_count = 0;
      sound_count = 0;
      light_count_nobody = 10*1000/5;// 十秒
      sound_count_nobody = 10*1000/5;// 十秒
      RTC.setTime(2022, 3, 11, 7, 0, 0);
    }
    
    
  }

  
  // 环境光
  if (!closed) {
    if (analogRead(LIGHT_PIN) < 100 ){   //读取引脚电压   A0光照
      light_count++;
    } else {
      light_count = 0;
    } 
    if (light_count > light_count_nobody) {
      // 关空调
      ir_send_close();
      closed = true;
      light_count = 0;
    }
    if (analogRead(SOUND_PIN) < 50){//A1引脚声音
      sound_count++;
    } else {
      sound_count = 0;
    }
    if (ask_flag){
      if (sound_count) {
        // 20秒询问一次
        if (((int)(time - ask_time)/1000)%(20) == 0 && ask_time_1 != time) {
            ask_time_1 = time;
             df1101s.playSpecFile(0);//播放0编号文件一次（询问语音）
         }
          // 1分钟无人回答
         if ((time - ask_time) > 1000*60*1) {
             ir_send_close();
             closed = true;
         }
      } else {
        ask_flag = false;
        df1101s.playSpecFile("/DF_REC/FILE0001.MP3");//播放指定编号文件一次（不关语音）
        //第一次询问之后半小时询问一次
        sound_count_nobody *= 3;  
      }
        
    }
  }

  
  int Hour = RTC.getTime(RTC.DS1307_HR);
  // 200ms 检测一次
  if ((time - last_time) > 200){
      //0点
      if (Hour >= 0 && Hour <6 && state != 0){
        // 关空调
        ir_send_close();
        ask_flag = false;
        sound_count_nobody = 10*60*1000/5;
        state = 0;
      }
      // 早上六点校正RTC以及重连WiFi
      if (Hour >= 6 && Hour <= 7 && state != 6){
        correct_time();
        state = 6;
        
      }
      // 上班时间
      if (Hour >= 8 && Hour < 19 && state != 12){
        closed = false;
        state = 12;
      }
      // 下班
      if (Hour >= 19 && Hour <= 23 && !closed && !ask_flag){
        Serial.print("sound count:");Serial.println(sound_count);
        // 10分钟无人
        if (sound_count > sound_count_nobody){
          df1101s.playSpecFile("/DF_REC/FILE0000.MP3");//播放指定编号文件一次（询问语音）
          ask_time = time;
          sound_count = 0;
          ask_flag = true;
        }
      }
      Serial.println("--");
      last_time = time;
      showtime();
  }
  
  delay(5);

}
